f32x8

Struct f32x8 

pub struct f32x8 { /* private fields */ }

Implementations

Source

impl f32x8

pub const ONE: f32x8

pub const HALF: f32x8

pub const ZERO: f32x8

pub const EPSILON: f32x8

pub const MIN: f32x8

pub const MIN_POSITIVE: f32x8

pub const MAX: f32x8

pub const NAN: f32x8

pub const INFINITY: f32x8

pub const NEG_INFINITY: f32x8

pub const E: f32x8

pub const FRAC_1_PI: f32x8

pub const FRAC_2_PI: f32x8

pub const FRAC_2_SQRT_PI: f32x8

pub const FRAC_1_SQRT_2: f32x8

pub const FRAC_PI_2: f32x8

pub const FRAC_PI_3: f32x8

pub const FRAC_PI_4: f32x8

pub const FRAC_PI_6: f32x8

pub const FRAC_PI_8: f32x8

pub const LN_2: f32x8

pub const LN_10: f32x8

pub const LOG2_E: f32x8

pub const LOG10_E: f32x8

pub const LOG10_2: f32x8

pub const LOG2_10: f32x8

pub const PI: f32x8

pub const SQRT_2: f32x8

pub const TAU: f32x8

Source

impl f32x8

pub const fn new(array: [f32; 8]) -> Self

pub fn simd_eq<Rhs>(self, other: Rhs) -> <Self as CmpEq<Rhs>>::Output
where Self: CmpEq<Rhs>,

Test if each element is equal to the corresponding element in other.

pub fn simd_ne<Rhs>(self, other: Rhs) -> <Self as CmpNe<Rhs>>::Output
where Self: CmpNe<Rhs>,

Test if each element is not equal to the corresponding element in other.

pub fn simd_lt<Rhs>(self, other: Rhs) -> <Self as CmpLt<Rhs>>::Output
where Self: CmpLt<Rhs>,

Test if each element is less than the corresponding element in other.

pub fn simd_gt<Rhs>(self, other: Rhs) -> <Self as CmpGt<Rhs>>::Output
where Self: CmpGt<Rhs>,

Test if each element is greater than the corresponding element in other.

pub fn simd_le<Rhs>(self, other: Rhs) -> <Self as CmpLe<Rhs>>::Output
where Self: CmpLe<Rhs>,

Test if each element is less than or equal to the corresponding element in other.

pub fn simd_ge<Rhs>(self, other: Rhs) -> <Self as CmpGe<Rhs>>::Output
where Self: CmpGe<Rhs>,

Test if each element is greater than or equal to the corresponding element in other.

pub fn blend(self, t: Self, f: Self) -> Self

pub fn abs(self) -> Self

pub fn signum(self) -> Self

pub fn floor(self) -> Self

pub fn ceil(self) -> Self

pub fn fast_max(self, rhs: Self) -> Self

Calculates the lanewise maximum of both vectors. This is a faster implementation than max, but it doesn’t specify any behavior if NaNs are involved.

pub fn max(self, rhs: Self) -> Self

Calculates the lanewise maximum of both vectors. This doesn’t match IEEE-754 and instead is defined as self < rhs ? rhs : self.

pub fn fast_min(self, rhs: Self) -> Self

Calculates the lanewise minimum of both vectors. This is a faster implementation than min, but it doesn’t specify any behavior if NaNs are involved.

pub fn min(self, rhs: Self) -> Self

Calculates the lanewise minimum of both vectors. If either lane is NaN, the other lane gets chosen. Use fast_min for a faster implementation that doesn’t handle NaNs.

pub fn clamp(self, min: Self, max: Self) -> Self

Restrict a value to a certain interval unless it is NaN.

If self is NaN, or min is NaN, or max is NaN, the result is NaN. If min > max, the result is min, since fast_max(min) dominates.

pub fn fast_clamp(self, min: Self, max: Self) -> Self

Restrict a value to a certain interval unless it is NaN.

Avoids NaN detection; same speed as the old clamp prior to IEEE 754-2019 compliance. Does not specify any behavior if NaNs are involved, and if min > max the result is unspecified.

pub fn midpoint(self, other: Self) -> Self

pub fn is_nan(self) -> Self

pub fn is_finite(self) -> Self

pub fn is_inf(self) -> Self

pub fn round(self) -> Self

pub fn fast_round_int(self) -> i32x8

Rounds each lane into an integer. This is a faster implementation than round_int, but it doesn’t handle out of range values or NaNs. For those values you get implementation defined behavior.

pub fn round_int(self) -> i32x8

Rounds each lane into an integer. This saturates out of range values and turns NaNs into 0. Use fast_round_int for a faster implementation that doesn’t handle out of range values or NaNs.

pub fn trunc(self) -> Self

pub fn fast_trunc_int(self) -> i32x8

Truncates each lane into an integer. This is a faster implementation than trunc_int, but it doesn’t handle out of range values or NaNs. For those values you get implementation defined behavior.

pub fn trunc_int(self) -> i32x8

Truncates each lane into an integer. This saturates out of range values and turns NaNs into 0. Use fast_trunc_int for a faster implementation that doesn’t handle out of range values or NaNs.

pub fn fract(self) -> Self

pub fn mul_add(self, m: Self, a: Self) -> Self

Performs a multiply-add operation: self * m + a

When hardware FMA support is available, this computes the result with a single rounding operation. Without FMA support, it falls back to separate multiply and add operations with two roundings.

Platform-specific behavior
  • On x86/x86_64 with AVX+FMA: Uses vfmadd (single rounding, best accuracy)
  • On x86/x86_64 with AVX only: Uses (self * m) + a (two roundings)
  • Other platforms: Delegates to f32x4 (may use NEON FMA or fallback)
Examples
let a = f32x8::from([1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0]);
let b = f32x8::from([2.0; 8]);
let c = f32x8::from([10.0; 8]);

let result = a.mul_add(b, c);

let expected = f32x8::from([12.0, 14.0, 16.0, 18.0, 20.0, 22.0, 24.0, 26.0]);
assert_eq!(result, expected);
pub fn mul_sub(self, m: Self, s: Self) -> Self

Performs a multiply-subtract operation: self * m - s

When hardware FMA support is available, this computes the result with a single rounding operation. Without FMA support, it falls back to separate multiply and subtract operations with two roundings.

Platform-specific behavior
  • On x86/x86_64 with AVX+FMA: Uses vfmsub (single rounding, best accuracy)
  • On x86/x86_64 with AVX only: Uses (self * m) - s (two roundings)
  • Other platforms: Delegates to f32x4 (may use NEON FMA or fallback)
Examples
let a = f32x8::from([10.0; 8]);
let b = f32x8::from([2.0; 8]);
let c = f32x8::from([5.0; 8]);

let result = a.mul_sub(b, c);

let expected = f32x8::from([15.0; 8]);
assert_eq!(result, expected);
pub fn mul_neg_add(self, m: Self, a: Self) -> Self

Performs a negative multiply-add operation: a - (self * m)

When hardware FMA support is available, this computes the result with a single rounding operation. Without FMA support, it falls back to separate operations with two roundings.

Platform-specific behavior
  • On x86/x86_64 with AVX+FMA: Uses vfnmadd (single rounding, best accuracy)
  • On x86/x86_64 with AVX only: Uses a - (self * m) (two roundings)
  • Other platforms: Delegates to f32x4 (may use NEON FMA or fallback)
Examples
let a = f32x8::from([3.0; 8]);
let b = f32x8::from([2.0; 8]);
let c = f32x8::from([10.0; 8]);

let result = a.mul_neg_add(b, c);

let expected = f32x8::from([4.0; 8]);
assert_eq!(result, expected);
pub fn mul_neg_sub(self, m: Self, s: Self) -> Self

Performs a negative multiply-subtract operation: -(self * m) - s

When hardware FMA support is available, this computes the result with a single rounding operation. Without FMA support, it falls back to separate operations with two roundings.

Platform-specific behavior
  • On x86/x86_64 with AVX+FMA: Uses vfnmsub (single rounding, best accuracy)
  • On x86/x86_64 with AVX only: Uses -(self * m) - s (two roundings)
  • Other platforms: Delegates to f32x4 (may use NEON FMA or fallback)
Examples
let a = f32x8::from([3.0; 8]);
let b = f32x8::from([2.0; 8]);
let c = f32x8::from([1.0; 8]);

let result = a.mul_neg_sub(b, c);

let expected = f32x8::from([-7.0; 8]);
assert_eq!(result, expected);
pub fn div_euclid(self, rhs: Self) -> Self

pub fn rem_euclid(self, rhs: Self) -> Self

pub fn flip_signs(self, signs: Self) -> Self

pub fn copysign(self, sign: Self) -> Self

pub fn asin_acos(self) -> (Self, Self)

pub fn asin(self) -> Self

pub fn acos(self) -> Self

pub fn atan(self) -> Self

pub fn atan2(self, x: Self) -> Self

pub fn sin_cos(self) -> (Self, Self)

pub fn sin(self) -> Self

pub fn cos(self) -> Self

pub fn tan(self) -> Self

pub fn sinh(self) -> Self

Calculates hyperbolic sine: (e^self - e^(-self))/2.

pub fn cosh(self) -> Self

Calculates hyperbolic cosine: (e^self + e^(-self))/2.

pub fn tanh(self) -> Self

Calculates hyperbolic tangent: sinh(self)/cosh(self).

pub fn cbrt(self) -> Self

Calculates the cube root: self^(1/3).

pub fn to_degrees(self) -> Self

pub fn to_radians(self) -> Self

pub fn recip(self) -> Self

pub fn recip_sqrt(self) -> Self

pub fn sqrt(self) -> Self

pub fn to_bitmask(self) -> u32

pub fn any(self) -> bool

pub fn all(self) -> bool

pub fn none(self) -> bool

pub fn exp(self) -> Self

Calculate the exponent of a packed f32x8

pub fn exp_m1(self) -> Self

Calculate e^self - 1 for each lane. Accurate even for very small values.

pub fn exp2(self) -> Self

Returns 2^self.

pub fn is_sign_positive(self) -> Self

Returns true for each element if it has a positive sign, including +0.0, NaNs with positive sign bit and positive infinity.

pub fn is_sign_negative(self) -> Self

Returns true for each element if it has a negative sign, including -0.0, NaNs with negative sign bit and negative infinity.

pub fn reduce_add(self) -> f32

horizontal add of all the elements of the vector

pub fn reduce_mul(self) -> f32

horizontal multiplication of all the elements of the vector

pub fn ln(self) -> Self

Natural log (ln(x))

pub fn ln_1p(self) -> Self

Calculate ln(1 + self) for each lane. Accurate even for very small values.

pub fn log2(self) -> Self

pub fn log10(self) -> Self

pub fn pow_f32x8(self, y: Self) -> Self

pub fn powf(self, y: f32) -> Self

pub fn transpose(data: [f32x8; 8]) -> [f32x8; 8]

Transpose matrix of 8x8 f32 matrix. Currently only accelerated on AVX.

pub fn to_array(self) -> [f32; 8]

pub fn as_array(&self) -> &[f32; 8]

pub fn as_mut_array(&mut self) -> &mut [f32; 8]

pub fn from_i32x8(v: i32x8) -> Self

pub fn sign_bit(self) -> Self

👎Deprecated since 1.4.0:

renamed to is_sign_negative

Returns true for each element if its sign bit is set.

If the sign bit is set, the result has all bits set, not just the sign bit. This has been renamed to is_sign_negative.

Source

impl f32x8

pub const fn splat(elem: f32) -> f32x8