i64x8

Struct i64x8 

pub struct i64x8 { /* private fields */ }
A SIMD vector with eight elements of type i64.

See the crate level documentation for more information about SIMD vectors.

Implementations

Source

impl i64x8

The following functionality exists for all SIMD vectors.

pub const fn new(array: [i64; 8]) -> Self

Converts an array to a SIMD vector.

pub const fn splat(elem: i64) -> Self

Constructs a new SIMD vector with all elements set to the given value.

pub const fn to_array(self) -> [i64; 8]

Converts a SIMD vector to an array.

pub const fn as_array(&self) -> &[i64; 8]

Returns an array reference containing the entire SIMD vector.

pub const fn as_mut_array(&mut self) -> &mut [i64; 8]

Returns a mutable array reference containing the entire SIMD vector.

pub fn simd_eq<Rhs>(self, other: Rhs) -> <Self as CmpEq<Rhs>>::Output
where Self: CmpEq<Rhs>,

Returns a mask that checks if each element of self is equal to the corresponding element of other.

pub fn simd_ne<Rhs>(self, other: Rhs) -> <Self as CmpNe<Rhs>>::Output
where Self: CmpNe<Rhs>,

Returns a mask that checks if each element of self is not equal to the corresponding element of other.

pub fn simd_lt<Rhs>(self, other: Rhs) -> <Self as CmpLt<Rhs>>::Output
where Self: CmpLt<Rhs>,

Returns a mask that checks if each element of self is less than the corresponding element of other.

pub fn simd_gt<Rhs>(self, other: Rhs) -> <Self as CmpGt<Rhs>>::Output
where Self: CmpGt<Rhs>,

Returns a mask that checks if each element of self is greater than the corresponding element of other.

pub fn simd_le<Rhs>(self, other: Rhs) -> <Self as CmpLe<Rhs>>::Output
where Self: CmpLe<Rhs>,

Returns a mask that checks if each element of self is less than or equal to the corresponding element of other.

pub fn simd_ge<Rhs>(self, other: Rhs) -> <Self as CmpGe<Rhs>>::Output
where Self: CmpGe<Rhs>,

Returns a mask that checks if each element of self is greater than or equal to the corresponding element of other.

pub fn bitselect(self, if_one: Self, if_zero: Self) -> Self

Bitwise selection.

For each bit of self:

  • If the bit is one, return the corresponding bit of if_one
  • If the bit is zero, return the corresponding bit of if_zero

If you know self is a mask, meaning each element is either all zeros or all ones, consider using select which is faster.

pub fn select(self, if_true: Self, if_false: Self) -> Self

Elementwise selection.

For each element of self:

  • If all bits are one, return the corresponding element of if_true
  • If all bits are zero, return the corresponding element of if_false

This function assumes self is a mask, meaning each element is either all zeros or all ones. For bitwise selection use bitselect.

pub fn to_bitmask(self) -> u32

Converts to a bitmask, where each bit is 1 if the element of self is true or 0 if the element of self is false.

Each bit of the output corresponds to an element of self. The least significant bit corresponds to the lowest element. Remaining bits are 0.

This function assumes self is a mask, meaning each element is either all zeros or all ones. If the input is not a mask, the result is unspecified.

returns the bit mask for each high bit set in the vector with the lowest lane being the lowest bit

pub fn any(self) -> bool

Returns true if any element of self is true.

This function assumes self is a mask, meaning each element is either all zeros or all ones. If the input is not a mask, the result is unspecified.

true if any high bits are set for any value in the vector

pub fn all(self) -> bool

Returns true if all elements of self are true.

This function assumes self is a mask, meaning each element is either all zeros or all ones. If the input is not a mask, the result is unspecified.

true if all high bits are set for every value in the vector

pub fn none(self) -> bool

Returns true if none of the elements of self are true.

This function assumes self is a mask, meaning each element is either all zeros or all ones. If the input is not a mask, the result is unspecified.

pub fn transpose(data: [i64x8; 8]) -> [i64x8; 8]

Transposes an array of SIMD vectors interpreted as a square matrix.

Currently this function is never accelerated.

pub fn blend(self, if_true: Self, if_false: Self) -> Self

👎Deprecated since 1.6.0:

split into select and bitselect functions

Elementwise selection.

For each element of self:

  • If all bits are one, return the corresponding element of if_true
  • If all bits are zero, return the corresponding element of if_false

Originally this function did not specify whether it supported per-bit selection, or if it assumed self is a mask, meaning each element is either all zeros or all ones (for better performance). Because of this, blend has been split into two new functions: select and bitselect.

Source

impl i64x8

The following functionality exists for all SIMD vectors of signed integers.

pub const ONE: Self

A SIMD vector with all elements set to 1.

pub const ZERO: Self

A SIMD vector with all elements set to 0.

pub const MAX: Self

A SIMD vector with all elements set to i64::MAX.

pub const MIN: Self

A SIMD vector with all elements set to i64::MIN.

pub const LANES: u16 = 8

The number of elements in this SIMD vector.

pub const BITS: u16

The size of this SIMD vector in bits.

pub fn max(self, rhs: Self) -> Self

Returns the maximum between each element of self and the corresponding element of other.

pub fn min(self, rhs: Self) -> Self

Returns the minimum between each element of self and the corresponding element of other.

pub fn clamp(self, min: Self, max: Self) -> Self

Clamps each element of self between the corresponding elements of min and max.

If min > max, the result is unspecified. Consider manually checking for that case.

pub fn reduce_add(self) -> i64

Reducing addition. Returns the sum of the vector’s elements.

Equivalent to self[0] + self[1] + ....

pub fn reduce_mul(self) -> i64

Reducing multiplication. Returns the product of the vector’s elements.

Equivalent to self[0] * self[1] * ....

pub fn reduce_max(self) -> i64

Reducing maximum. Returns the maximum of the vector’s elements.

Equivalent to self[0].max(self[1].max(...)).

pub fn reduce_min(self) -> i64

Reducing minimum. Returns the minimum of the vector’s elements.

Equivalent to self[0].min(self[1].min(...)).

pub const fn cast_unsigned(self) -> u64x8

Returns the bit patterns of self reinterpreted as unsigned integers of the same size.

pub fn unbounded_shl(self, rhs: u64x8) -> Self

Shifts left each element of self by the corresponding element of rhs, without bounding rhs.

If rhs is larger than or equal to the number of bits in i64, the entire value is shifted out, and 0 is returned.

This is different from the standard operator, which behaves like wrapping_shl. For most targets, unbounded_shl is faster than the standard operator.

If you intend to shift all elements by the same value, consider using unbounded_shl_scalar which is faster.

pub fn unbounded_shl_scalar(self, rhs: u32) -> Self

Shifts left each element of self by the uniform scalar rhs, without bounding rhs.

If rhs is larger than or equal to the number of bits in i64, the entire value is shifted out, and 0 is returned.

This is different from the standard operator, which behaves like wrapping_shl. For most targets, unbounded_shl_scalar is faster than the standard operator.

This function is faster than self.unbounded_shl(splat(rhs)) because it has special hardware support.

pub fn unbounded_shr(self, rhs: u64x8) -> Self

Shifts right each element of self by the corresponding element of rhs, without bounding rhs.

If rhs is larger than or equal to the number of bits in i64, the entire value is shifted out, which yields 0 for a positive number, and -1 for a negative number.

This is different from the standard operator, which behaves like wrapping_shr. For most targets, unbounded_shr is faster than the standard operator.

If you intend to shift all elements by the same value, consider using unbounded_shr_scalar which is faster.

pub fn unbounded_shr_scalar(self, rhs: u32) -> Self

Shifts right each element of self by the uniform scalar rhs, without bounding rhs.

If rhs is larger than or equal to the number of bits in i64, the entire value is shifted out, which yields 0 for a positive number, and -1 for a negative number.

This is different from the standard operator, which behaves like wrapping_shr. For most targets, unbounded_shr_scalar is faster than the standard operator.

This function is faster than self.unbounded_shr(splat(rhs)) because it has special hardware support.

pub fn saturating_add(self, rhs: Self) -> Self

Saturating integer addition. Computes self + rhs, saturating at the numeric bounds instead of overflowing.

pub fn saturating_sub(self, rhs: Self) -> Self

Saturating integer subtraction. Computes self - rhs, saturating at the numeric bounds instead of overflowing.

pub fn saturating_mul(self, rhs: Self) -> Self

Saturating integer multiplication. Computes self * rhs, saturating at the numeric bounds instead of overflowing.

pub fn saturating_div(self, rhs: Self) -> Self

Saturating integer division. Computes self / rhs, saturating at the numeric bounds instead of overflowing.

Note that because division has no hardware support, this operation is very slow and should be avoided if possible.

Panics

Panics if any element of rhs is zero.

pub fn unsigned_abs(self) -> u64x8

Computes the absolute value of each input element, returned as an unsigned integer in order to avoid wrapping.

pub fn overflowing_add(self, rhs: Self) -> (Self, Self)

Returns self + rhs and whether an overflow occured.

Returns a tuple with:

  • The addition (returns the wrapped value if an overflow occured)
  • A mask indicating whether an overflow occured
pub fn overflowing_sub(self, rhs: Self) -> (Self, Self)

Returns self - rhs and whether an overflow occured.

Returns a tuple with:

  • The subtraction (returns the wrapped value if an overflow occured)
  • A mask indicating whether an overflow occured
pub fn overflowing_mul(self, rhs: Self) -> (Self, Self)

Returns self * rhs and whether an overflow occured.

Returns a tuple with:

  • The multiplication (returns the wrapped value if an overflow occured)
  • A mask indicating whether an overflow occured
pub fn overflowing_div(self, rhs: Self) -> (Self, Self)

Returns self / rhs and whether an overflow occured.

Returns a tuple with:

  • The division (returns self if an overflow occured)
  • A mask indicating whether an overflow occured

Note that because division has no hardware support, this operation is very slow and should be avoided if possible.

pub fn overflowing_rem(self, rhs: Self) -> (Self, Self)

Returns self % rhs and whether an overflow occured.

Returns a tuple with:

  • The remainder (returns zero if an overflow occured)
  • A mask indicating whether an overflow occured

Note that because division has no hardware support, this operation is very slow and should be avoided if possible.

pub fn mul_keep_low_high(self, rhs: Self) -> (u64x8, i64x8)

Computes self * rhs, producing intermediate 128-bit integers, then returns their low 64-bit parts and high 64-bit parts in two seperate SIMD vectors.

This function is different from widening_mul, which returns a single SIMD vector of a larger integer type, instead of two seperate SIMD vectors for low and high parts. Also note that while mul_keep_low_high exists for all types, widening_mul does not exist for types with no wider variant (e.g., for i32x16 because there is no i64x16).

pub fn mul_keep_high(self, rhs: Self) -> Self

Computes self * rhs, producing intermediate 128-bit integers, then returns their high 64-bit parts.

pub fn abs(self) -> Self

Returns the absolute value of each input element.

pub fn signum(self) -> Self

Returns numbers representing the sign of each element.

  • 0 if the element is zero
  • 1 if the element is positive
  • -1 if the element is negative
pub fn is_positive(self) -> Self

Returns a mask that is true for each positive element, and false if it is zero or negative.

pub fn is_negative(self) -> Self

Returns a mask that is true for each negative element, and false if it is zero or positive.

Source

impl i64x8

The following functionality exists only for i64x8, or only for particular types inconsistently.

pub fn round_float(self) -> f64x8

Converts each element from i64 to f64.