pub struct i64x8 { /* private fields */ }Expand description
A SIMD vector with eight elements of type i64.
See the crate level documentation for more information about SIMD vectors.
Implementations
Sourceimpl i64x8
The following functionality exists for all SIMD vectors.
impl i64x8
The following functionality exists for all SIMD vectors.
Sourcepub const fn splat(elem: i64) -> Self
pub const fn splat(elem: i64) -> Self
Constructs a new SIMD vector with all elements set to the given value.
Sourcepub const fn as_array(&self) -> &[i64; 8]
pub const fn as_array(&self) -> &[i64; 8]
Returns an array reference containing the entire SIMD vector.
Sourcepub const fn as_mut_array(&mut self) -> &mut [i64; 8]
pub const fn as_mut_array(&mut self) -> &mut [i64; 8]
Returns a mutable array reference containing the entire SIMD vector.
Sourcepub fn simd_eq<Rhs>(self, other: Rhs) -> <Self as CmpEq<Rhs>>::Outputwhere
Self: CmpEq<Rhs>,
pub fn simd_eq<Rhs>(self, other: Rhs) -> <Self as CmpEq<Rhs>>::Outputwhere
Self: CmpEq<Rhs>,
Returns a mask that checks if each element of self is equal to the
corresponding element of other.
Sourcepub fn simd_ne<Rhs>(self, other: Rhs) -> <Self as CmpNe<Rhs>>::Outputwhere
Self: CmpNe<Rhs>,
pub fn simd_ne<Rhs>(self, other: Rhs) -> <Self as CmpNe<Rhs>>::Outputwhere
Self: CmpNe<Rhs>,
Returns a mask that checks if each element of self is not equal to
the corresponding element of other.
Sourcepub fn simd_lt<Rhs>(self, other: Rhs) -> <Self as CmpLt<Rhs>>::Outputwhere
Self: CmpLt<Rhs>,
pub fn simd_lt<Rhs>(self, other: Rhs) -> <Self as CmpLt<Rhs>>::Outputwhere
Self: CmpLt<Rhs>,
Returns a mask that checks if each element of self is less than
the corresponding element of other.
Sourcepub fn simd_gt<Rhs>(self, other: Rhs) -> <Self as CmpGt<Rhs>>::Outputwhere
Self: CmpGt<Rhs>,
pub fn simd_gt<Rhs>(self, other: Rhs) -> <Self as CmpGt<Rhs>>::Outputwhere
Self: CmpGt<Rhs>,
Returns a mask that checks if each element of self is greater than
the corresponding element of other.
Sourcepub fn simd_le<Rhs>(self, other: Rhs) -> <Self as CmpLe<Rhs>>::Outputwhere
Self: CmpLe<Rhs>,
pub fn simd_le<Rhs>(self, other: Rhs) -> <Self as CmpLe<Rhs>>::Outputwhere
Self: CmpLe<Rhs>,
Returns a mask that checks if each element of self is less than or
equal to the corresponding element of other.
Sourcepub fn simd_ge<Rhs>(self, other: Rhs) -> <Self as CmpGe<Rhs>>::Outputwhere
Self: CmpGe<Rhs>,
pub fn simd_ge<Rhs>(self, other: Rhs) -> <Self as CmpGe<Rhs>>::Outputwhere
Self: CmpGe<Rhs>,
Returns a mask that checks if each element of self is greater than
or equal to the corresponding element of other.
Sourcepub fn select(self, if_true: Self, if_false: Self) -> Self
pub fn select(self, if_true: Self, if_false: Self) -> Self
Elementwise selection.
For each element of self:
- If all bits are one, return the corresponding element of
if_true - If all bits are zero, return the corresponding element of
if_false
This function assumes self is a mask, meaning each element is
either all zeros or all ones. For bitwise selection use bitselect.
Sourcepub fn to_bitmask(self) -> u32
pub fn to_bitmask(self) -> u32
Converts to a bitmask, where each bit is 1 if the element of self
is true or 0 if the element of self is false.
Each bit of the output corresponds to an element of self. The least
significant bit corresponds to the lowest element. Remaining bits are
0.
This function assumes self is a mask, meaning each element is
either all zeros or all ones. If the input is not a mask, the result
is unspecified.
returns the bit mask for each high bit set in the vector with the lowest lane being the lowest bit
Sourcepub fn any(self) -> bool
pub fn any(self) -> bool
Returns true if any element of self is true.
This function assumes self is a mask, meaning each element is
either all zeros or all ones. If the input is not a mask, the result
is unspecified.
true if any high bits are set for any value in the vector
Sourcepub fn all(self) -> bool
pub fn all(self) -> bool
Returns true if all elements of self are true.
This function assumes self is a mask, meaning each element is
either all zeros or all ones. If the input is not a mask, the result
is unspecified.
true if all high bits are set for every value in the vector
Sourcepub fn none(self) -> bool
pub fn none(self) -> bool
Returns true if none of the elements of self are true.
This function assumes self is a mask, meaning each element is
either all zeros or all ones. If the input is not a mask, the result
is unspecified.
Sourcepub fn transpose(data: [i64x8; 8]) -> [i64x8; 8]
pub fn transpose(data: [i64x8; 8]) -> [i64x8; 8]
Transposes an array of SIMD vectors interpreted as a square matrix.
Currently this function is never accelerated.
Sourcepub fn blend(self, if_true: Self, if_false: Self) -> Self
👎Deprecated since 1.6.0: split into select and bitselect functions
pub fn blend(self, if_true: Self, if_false: Self) -> Self
split into select and bitselect functions
Elementwise selection.
For each element of self:
- If all bits are one, return the corresponding element of
if_true - If all bits are zero, return the corresponding element of
if_false
Originally this function did not specify whether it supported per-bit
selection, or if it assumed self is a mask, meaning each element
is either all zeros or all ones (for better performance). Because of
this, blend has been split into two new functions: select and
bitselect.
Sourceimpl i64x8
The following functionality exists for all SIMD vectors of signed
integers.
impl i64x8
The following functionality exists for all SIMD vectors of signed integers.
Sourcepub fn max(self, rhs: Self) -> Self
pub fn max(self, rhs: Self) -> Self
Returns the maximum between each element of self and the
corresponding element of other.
Sourcepub fn min(self, rhs: Self) -> Self
pub fn min(self, rhs: Self) -> Self
Returns the minimum between each element of self and the
corresponding element of other.
Sourcepub fn clamp(self, min: Self, max: Self) -> Self
pub fn clamp(self, min: Self, max: Self) -> Self
Clamps each element of self between the corresponding elements of
min and max.
If min > max, the result is unspecified. Consider manually checking
for that case.
Sourcepub fn reduce_add(self) -> i64
pub fn reduce_add(self) -> i64
Reducing addition. Returns the sum of the vector’s elements.
Equivalent to self[0] + self[1] + ....
Sourcepub fn reduce_mul(self) -> i64
pub fn reduce_mul(self) -> i64
Reducing multiplication. Returns the product of the vector’s elements.
Equivalent to self[0] * self[1] * ....
Sourcepub fn reduce_max(self) -> i64
pub fn reduce_max(self) -> i64
Reducing maximum. Returns the maximum of the vector’s elements.
Equivalent to self[0].max(self[1].max(...)).
Sourcepub fn reduce_min(self) -> i64
pub fn reduce_min(self) -> i64
Reducing minimum. Returns the minimum of the vector’s elements.
Equivalent to self[0].min(self[1].min(...)).
Sourcepub const fn cast_unsigned(self) -> u64x8
pub const fn cast_unsigned(self) -> u64x8
Returns the bit patterns of self reinterpreted as unsigned integers
of the same size.
Sourcepub fn unbounded_shl(self, rhs: u64x8) -> Self
pub fn unbounded_shl(self, rhs: u64x8) -> Self
Shifts left each element of self by the corresponding element of
rhs, without bounding rhs.
If rhs is larger than or equal to the number of bits in i64,
the entire value is shifted out, and 0 is returned.
This is different from the standard operator, which behaves like
wrapping_shl. For most targets, unbounded_shl is faster than the
standard operator.
If you intend to shift all elements by the same value, consider using
unbounded_shl_scalar which is faster.
Sourcepub fn unbounded_shl_scalar(self, rhs: u32) -> Self
pub fn unbounded_shl_scalar(self, rhs: u32) -> Self
Shifts left each element of self by the uniform scalar rhs,
without bounding rhs.
If rhs is larger than or equal to the number of bits in i64,
the entire value is shifted out, and 0 is returned.
This is different from the standard operator, which behaves like
wrapping_shl. For most targets, unbounded_shl_scalar is faster
than the standard operator.
This function is faster than self.unbounded_shl(splat(rhs)) because
it has special hardware support.
Sourcepub fn unbounded_shr(self, rhs: u64x8) -> Self
pub fn unbounded_shr(self, rhs: u64x8) -> Self
Shifts right each element of self by the corresponding element of
rhs, without bounding rhs.
If rhs is larger than or equal to the number of bits in i64,
the entire value is shifted out, which yields 0 for a positive
number, and -1 for a negative number.
This is different from the standard operator, which behaves like
wrapping_shr. For most targets, unbounded_shr is faster than the
standard operator.
If you intend to shift all elements by the same value, consider using
unbounded_shr_scalar which is faster.
Sourcepub fn unbounded_shr_scalar(self, rhs: u32) -> Self
pub fn unbounded_shr_scalar(self, rhs: u32) -> Self
Shifts right each element of self by the uniform scalar rhs,
without bounding rhs.
If rhs is larger than or equal to the number of bits in i64,
the entire value is shifted out, which yields 0 for a positive
number, and -1 for a negative number.
This is different from the standard operator, which behaves like
wrapping_shr. For most targets, unbounded_shr_scalar is faster
than the standard operator.
This function is faster than self.unbounded_shr(splat(rhs)) because
it has special hardware support.
Sourcepub fn saturating_add(self, rhs: Self) -> Self
pub fn saturating_add(self, rhs: Self) -> Self
Saturating integer addition. Computes self + rhs, saturating at the
numeric bounds instead of overflowing.
Sourcepub fn saturating_sub(self, rhs: Self) -> Self
pub fn saturating_sub(self, rhs: Self) -> Self
Saturating integer subtraction. Computes self - rhs, saturating at
the numeric bounds instead of overflowing.
Sourcepub fn saturating_mul(self, rhs: Self) -> Self
pub fn saturating_mul(self, rhs: Self) -> Self
Saturating integer multiplication. Computes self * rhs, saturating
at the numeric bounds instead of overflowing.
Sourcepub fn saturating_div(self, rhs: Self) -> Self
pub fn saturating_div(self, rhs: Self) -> Self
Saturating integer division. Computes self / rhs, saturating at the
numeric bounds instead of overflowing.
Note that because division has no hardware support, this operation is very slow and should be avoided if possible.
Panics
Panics if any element of rhs is zero.
Sourcepub fn unsigned_abs(self) -> u64x8
pub fn unsigned_abs(self) -> u64x8
Computes the absolute value of each input element, returned as an unsigned integer in order to avoid wrapping.
Sourcepub fn overflowing_add(self, rhs: Self) -> (Self, Self)
pub fn overflowing_add(self, rhs: Self) -> (Self, Self)
Returns self + rhs and whether an overflow occured.
Returns a tuple with:
- The addition (returns the wrapped value if an overflow occured)
- A mask indicating whether an overflow occured
Sourcepub fn overflowing_sub(self, rhs: Self) -> (Self, Self)
pub fn overflowing_sub(self, rhs: Self) -> (Self, Self)
Returns self - rhs and whether an overflow occured.
Returns a tuple with:
- The subtraction (returns the wrapped value if an overflow occured)
- A mask indicating whether an overflow occured
Sourcepub fn overflowing_mul(self, rhs: Self) -> (Self, Self)
pub fn overflowing_mul(self, rhs: Self) -> (Self, Self)
Returns self * rhs and whether an overflow occured.
Returns a tuple with:
- The multiplication (returns the wrapped value if an overflow occured)
- A mask indicating whether an overflow occured
Sourcepub fn overflowing_div(self, rhs: Self) -> (Self, Self)
pub fn overflowing_div(self, rhs: Self) -> (Self, Self)
Returns self / rhs and whether an overflow occured.
Returns a tuple with:
- The division (returns
selfif an overflow occured) - A mask indicating whether an overflow occured
Note that because division has no hardware support, this operation is very slow and should be avoided if possible.
Sourcepub fn overflowing_rem(self, rhs: Self) -> (Self, Self)
pub fn overflowing_rem(self, rhs: Self) -> (Self, Self)
Returns self % rhs and whether an overflow occured.
Returns a tuple with:
- The remainder (returns zero if an overflow occured)
- A mask indicating whether an overflow occured
Note that because division has no hardware support, this operation is very slow and should be avoided if possible.
Sourcepub fn mul_keep_low_high(self, rhs: Self) -> (u64x8, i64x8)
pub fn mul_keep_low_high(self, rhs: Self) -> (u64x8, i64x8)
Computes self * rhs, producing intermediate 128-bit integers, then returns their low 64-bit parts and high 64-bit parts in two seperate SIMD vectors.
This function is different from widening_mul, which returns a single
SIMD vector of a larger integer type, instead of two seperate SIMD
vectors for low and high parts. Also note that while
mul_keep_low_high exists for all types, widening_mul does not
exist for types with no wider variant (e.g., for i32x16 because
there is no i64x16).
Sourcepub fn mul_keep_high(self, rhs: Self) -> Self
pub fn mul_keep_high(self, rhs: Self) -> Self
Computes self * rhs, producing intermediate 128-bit integers, then returns their high 64-bit parts.
Sourcepub fn signum(self) -> Self
pub fn signum(self) -> Self
Returns numbers representing the sign of each element.
0if the element is zero1if the element is positive-1if the element is negative
Sourcepub fn is_positive(self) -> Self
pub fn is_positive(self) -> Self
Returns a mask that is true for each positive element, and false if it is zero or negative.
Sourcepub fn is_negative(self) -> Self
pub fn is_negative(self) -> Self
Returns a mask that is true for each negative element, and false if it is zero or positive.